/**************************************************************************/
/*                           File: align_main.cc                          */
/**************************************************************************/
/* Project: Proper Noun Pronunciations Dictionary Alignment               */
/* Author: Neeraj Deshmukh                                                */
/**************************************************************************/
 
/*========================================================================
  This file defines the main function used in the alignment of the proper 
  nouns dictionary
  ========================================================================*/

/*------------------------------------------------------------------------
  include files
  ------------------------------------------------------------------------*/
#include "dict_align.h"

/*------------------------------------------------------------------------
  forward declarations
  ------------------------------------------------------------------------*/
class Back_Pointer;

/*========================================================================
  function definition for the alignment of the dictionary
  ========================================================================*/
main (int argc, char **argv)
{
  // ensure correct call
  //
  if (argc != 2)
    {
      fprintf (stderr, "Usage is align_dict.exe <list_data_file>\n");
      exit (-1);
    }

  // open the input file containg the list of dictiaonary files
  //
  FILE *listFile = open_file_cc (argv[1], "r");
  
  // define strings required to store filenames and data
  //
  String dict_name   = new char[80];
  String out_name    = new char[80];
  String spell_word  = new char[50];
  String *pron_word;
  
  // define integers to keep count and score
  //
  int ph = 0, b_ph = 0;
  int num_phonemes = 0;
  int word_length  = 0;
  int score_this   = 0;
  int score_back   = 0;
  int blank_score  = get_score_cc (' ', "_");
  
  // define data structures that keep track of path
  //
  Back_Pointer *this_path;
  Back_Pointer *other_path;
  Back_Pointer *temp_char;

  // read the dictionary file name and open it
  //
  while (fscanf (listFile, "%s", dict_name) != EOF)
    {
      FILE *dictFile = open_file_cc (dict_name, "r");
      
      // open a corresponding output file
      //
      strcpy (out_name, dict_name);
      strcat (out_name, ".align");
      FILE *outFile = open_file_cc (out_name, "w");
      
      // read the dictionary entries one by one to start alignment
      //
      while (fscanf (dictFile, "%[^\t]", spell_word) != EOF)
	{
	  word_length = strlen (spell_word);
	  
	  // read the pronunciation entry
	  //
	  pron_word = read_pron_cc (dictFile, &num_phonemes);
	  
	  // start the alignment process
	  //
	  this_path = NULL;
	  other_path = NULL;
	  temp_char = NULL;
	  
	  // do this only if the spelling is not aligned already
	  //
	  if (word_length > num_phonemes)
	    {
	      // keep track of phonemes
	      //
	      ph   = 0;
	      b_ph = 0;
	      
	      // iterate through all the letters in the word spelling
	      //
	      for (int i = 0; i < word_length; i++)
		{
		  // initialize the paths
		  //
		  if (i == 0)
		    {
		      score_this = get_score_cc (spell_word[i], pron_word[ph]);
		      
		      // create a new backpointer
		      //
		      if (score_this >= blank_score)
			{
			  temp_char = new Back_Pointer (pron_word[ph], score_this, this_path);
			  ph ++;
			}
		      else
			temp_char = new Back_Pointer ("_", blank_score, this_path);
		      
		      // put it at the head of the list
		      //
		      this_path  = temp_char;
		      other_path = temp_char;
		      temp_char  = NULL;
		    }
		  
		  // for the second letter onwards we have two possibilities
		  //
		  else if (ph < num_phonemes)
		    {
		      // if all phonemes have not been run through
		      //
		      if (ph > 1) 
			b_ph = ph - 1;
		      
		      // check score of phoneme corresponding to last letter
		      //
		      score_back = get_score_cc (spell_word[i], pron_word[b_ph]);
		      
		      // check score of phoneme corresponding to this letter
		      //
		      score_this = get_score_cc (spell_word[i], pron_word[ph]);
		      
		      // grow a new backpointer with a better path
		      //
		      if ((this_path->get_score_cc () + blank_score) > 
			  (other_path->get_score_cc () + score_back))
			temp_char  = new Back_Pointer ("_", blank_score, this_path);
		      
		      else if (score_back >= blank_score)
			temp_char  = new Back_Pointer (pron_word[b_ph], score_back, other_path);
		      
		      else
			temp_char  = new Back_Pointer ("_", blank_score, other_path);
		      other_path = temp_char;
		      
		      // grow the main path
		      //
		      if (score_this > blank_score)
			{
			  temp_char  = new Back_Pointer (pron_word[ph], score_this, this_path);
			  ph ++;
			}
		      else
			temp_char  = new Back_Pointer ("_", blank_score, this_path);
		      this_path  = temp_char;
		      temp_char  = NULL;
		    }
		  
		  else if (b_ph == (num_phonemes - 2))
		    {
		      b_ph ++;
		      score_back = get_score_cc (spell_word[i], pron_word[b_ph]);
		      if (score_back >= blank_score)
			temp_char  = new Back_Pointer (pron_word[b_ph], score_back, other_path);
		      else
			temp_char  = new Back_Pointer ("_", blank_score, other_path);
		      other_path = temp_char;
		      
		      temp_char  = new Back_Pointer ("_", blank_score, this_path);
		      this_path  = temp_char;
		      temp_char  = NULL;
		    }
		  
		  else
		    {
		      // now all the rest are blank phones
		      //
		      temp_char = new Back_Pointer ("_", blank_score, this_path);
		      
		      // put it at the head of the list
		      //
		      this_path = temp_char;
		      temp_char = NULL;
		      
		      temp_char = new Back_Pointer ("_", blank_score, other_path);
		      
		      // put it at the head of the list
		      //
		      other_path = temp_char;
		      temp_char = NULL;
		    } 
		}
	      
	      // find which path is better and correspondingly write outfile
	      //
	      if (this_path->get_score_cc () >= other_path->get_score_cc ())
		{
		  write_word_cc (outFile, spell_word);
		  this_path->backtrace_cc (outFile, word_length);
		}
	      
	      else
		{
		  write_word_cc (outFile, spell_word);
		  other_path->backtrace_cc (outFile, word_length);
		}

	      // free memory
	      //
	      delete this_path;
	      delete other_path;
	    }
	  
	  else
	    {
	      // otherwise simply write back the entry
	      //
	      write_word_cc (outFile, spell_word);
	      for (int i = 0; i < num_phonemes; i ++)
		fprintf (outFile, " %s", pron_word[i]);
	      fprintf (outFile, "\n");
	    }
	  
	  // free memory
	  //
	  delete [] pron_word;
	}
      
      // close the input/output files
      //
      fclose (dictFile);
      fclose (outFile);
    }
  
  // free memory in strings
  //
  fclose (listFile);
  delete [] dict_name;
  delete [] out_name;
  delete [] spell_word;
}
