/**************************************************************************/
/*                           File: dict_align.h                           */
/**************************************************************************/
/* Project: Proper Noun Pronunciations Dictionary Alignment               */
/* Author: Neeraj Deshmukh                                                */
/**************************************************************************/
 
/*========================================================================
  This file declares the class used in the alignment of the proper nouns
  dictionary
  ========================================================================*/

/*------------------------------------------------------------------------
  make sure definitions are only made once
  ------------------------------------------------------------------------*/
#ifndef __ISIP_DICT_ALIGN
#define __ISIP_DICT_ALIGN

/*------------------------------------------------------------------------
  system and ISIP include files
  ------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <math.h>

/*------------------------------------------------------------------------
  declaration of global data types
  ------------------------------------------------------------------------*/
typedef char *String;

/*------------------------------------------------------------------------
  declaration of global functions
  ------------------------------------------------------------------------*/
extern String *read_pron_cc (FILE *inf, int *num_phones);
extern FILE   *open_file_cc (String name, String mode);
extern int    get_score_cc (char letter = ' ', String phoneme = "");
extern void   write_word_cc (FILE *outf, String word);

/*========================================================================
  Back_Pointer
  class declaration for the back pointer that keeps track of the best path
  ========================================================================*/
class Back_Pointer {

  // members
  //
protected:
  
  String         this_phoneme_d;
  int            best_score_d;
  Back_Pointer   *prev_phoneme_d;

  // methods
  //
public:

  /*----------------------------
    constructors and destructors
    ----------------------------*/
  Back_Pointer (String phoneme = "", int score = 0, Back_Pointer *prev = NULL);
  Back_Pointer (Back_Pointer *new_bkptr);
  ~Back_Pointer ();
  
  /*----------------------
    input / output methods
    ----------------------*/
  void    put_phone_cc (String phoneme, int score);    // to store a phoneme
  int     get_score_cc ();                             // get best path score
  String  get_phone_cc ();                             // get the phoneme
  
  /*--------------------
    backtrace and output
    --------------------*/
  void backtrace_cc (FILE *outf, int length);  // backtrace and write to file
};

#endif
